/******************************************************************************
*  --- COVARIANCE ESTIMATOR ---                                               *
*                                                                             *
*  Copyright 2009 Bernhard Zeisl, Pierre Fite Georgel                         *
*                                                                             *
*  This program is free software: you can redistribute it and/or modify it    *
*  under the terms of the GNU General Public License as published by the Free *
*  Software Foundation, version 3 of the License.                             *
*  This program is distributed in the hope that it will be useful, but        *
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY *
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License    *
*  (LISENCE.txt) and the disclaimer (DISCLAIMER.txt) for more details.        *
*                                                                             *
*  The software was written by Bernhard Zeisl <bernhard.zeisl@mytum.de and    *
*  Pierre Fite Georgel <pierre.georgel@gmail.com>.                            *
*  For more information please contact the project website at:                *
*  http://campar.in.tum.de/Main/CovarianceEstimator                           *
*                                                                             *
*  The software package makes use of three additional libraries:              *
*  - The openCV computer vision library http://opencv.willowgarage.com/       *
*    published under the open source BSD license.                             *
*  - The SIFT implementation from Rob Hess <hess@eecs.oregonstate.edu>.       *
*    please see http://web.engr.oregonstate.edu/~hess/                        *
*    The package is distributed under GNU GPL v2 exept for the SIFT featur    *
*    detection code. For more information on SIFT licensing see LISENCE.ubc.  *
*  - The openSURF library from http://code.google.com/p/opensurf1/            *
*    The package is distributed under GNU GPL v3.                             *
*                                                                             *
*  If you use our software, please cite following publication:                *
*  Zeisl, B. et.al. "Estimation of Location Uncertainty for Scale Invariant   *
*    Feature Points", Proc. of the British Maschine Vision Conference (BMCV)  *
*    2009                                                                     *
*                                                                             *
*  @INPROCEEDINGS{ zeisl09elu,                                                *
*    AUTHOR={Bernhard Zeisl and Pierre Fite Georgel and Florian Schweiger and *
*            Eckehard Steinbach and Nassir Navab},                            *
*    TITEL={{Estimation of Location Uncertainty for Scale Invariant Feature   *
*            Points}},                                                        *
*    BOOKTITLE={Proc. of the British Maschine Vision Conference (BMCV)},      *
*    YEAR={2009}                                                              *
*  }                                                                          *
*                                                                             *
*******************************************************************************/

#pragma once

#include <iostream>
#include <string>

#include "definitions.h"

using namespace std;


class CovEstArgumentEvaluator {

public:
	/*** Constructor, Destructor ***/
	CovEstArgumentEvaluator() :
		imgDir("./"), imgFile(NULL), keyFileEnding(NULL),
		verbose(true), saveCov(false), showCov(false),
		decType(-1) {};

	CovEstArgumentEvaluator(int argc, char* argv[]) :
		decType(-1) {
		evaluate(argc, argv);
	};

	/*** Methods ***/

	void evaluate(int argc, char* argv[]);


	/*** Member variables ***/
	char* imgDir, * imgFile, * keyFileEnding;
	char imgFileStart[25];
	string keyFile;
	bool verbose, saveCov, showCov;
	int decType;

};


/*** Implementations - quick and dirty ***/

void CovEstArgumentEvaluator::evaluate(int argc, char* argv[]) {
	// Information on usage
	if(argc == 1) {
		cout << "USAGE:\n"
			<< "  ./covEstimate -t SIFT|SURF -i <img> [OPTIONS]\n"
			<< endl
			<< "REQUIRED PARAMETERS:\n"
			<< "  -t <type>    Detector type, SURF or SIFT\n"
			 << "  -i <img>     Image file to process.\n"
			 << endl
			 << "OPTIONS:\n"
			 << "  -k <file>    File ending of the keypoint file to load. The name itself\n"
			 << "               is according to the name of the image.\n"
			 << "               (default is " << FE_KEYS_SIFT << "|" << FE_KEYS_SURF << ")\n"
			 << "  -d <dir>     The directory containing the image file (default ist ./)\n"
			 << "  -q           Quite prossing. Don't print status information.\n"
			 << "  -show        Display the estimated uncertainty for feature points in the\n"
			 << "               image, each as ellipse according to the covariance matrix.\n"
			 << "  -save        Draw the covariance in the image and save to file.\n";

		exit(0);
	}

	// parse arguments
	int arg = 0;
	while (++arg < argc) { 
		if (! strcmp(argv[arg], "-t")) {
			string decTypeName( argv[++arg] );
			if(! strcmp(decTypeName.c_str(), "SIFT") )
				decType = DETECTOR_SIFT;
			else if(! strcmp(decTypeName.c_str(), "SURF") )
				decType = DETECTOR_SURF;
			else {
				cout << "COV Error: Unknown detector/descriptor type." << endl;
				exit(1);
			}
		}
		else if (! strcmp(argv[arg], "-d"))
			imgDir = argv[++arg];
		else if (! strcmp(argv[arg], "-i"))
			imgFile = argv[++arg]; 
		else if (! strcmp(argv[arg], "-k"))
			keyFileEnding = argv[++arg];
		else if (! strcmp(argv[arg], "-q"))
			verbose = false;
		else if (! strcmp(argv[arg], "-save"))
			saveCov = true;
		else if (! strcmp(argv[arg], "-show"))
			showCov = true;
		else
			cout << "Warning: Skipping unknown parameter " << argv[arg] << "." << endl;
	}

	if(imgFile == NULL) {
		cout << "COV Error: No image file specified." << endl;
		exit(1);
	}
	else {
		int i = 0;
		while(imgFile[i] != '.') imgFileStart[i] = imgFile[i++];
		imgFileStart[i] = 0;
	}

	if(keyFileEnding == NULL)
		switch (decType) {
			case DETECTOR_SIFT:
				keyFileEnding = FE_KEYS_SIFT;
				break;
			case DETECTOR_SURF:
				keyFileEnding = FE_KEYS_SURF;
				break;
		}
	
	keyFile.clear();
	keyFile.append(imgFileStart).append(keyFileEnding);

}