/******************************************************************************
*  --- COVARIANCE ESTIMATOR ---                                               *
*                                                                             *
*  Copyright 2009 Bernhard Zeisl, Pierre Fite Georgel                         *
*                                                                             *
*  This program is free software: you can redistribute it and/or modify it    *
*  under the terms of the GNU General Public License as published by the Free *
*  Software Foundation, version 3 of the License.                             *
*  This program is distributed in the hope that it will be useful, but        *
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY *
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License    *
*  (LISENCE.txt) and the disclaimer (DISCLAIMER.txt) for more details.        *
*                                                                             *
*  The software was written by Bernhard Zeisl <bernhard.zeisl@mytum.de and    *
*  Pierre Fite Georgel <pierre.georgel@gmail.com>.                            *
*  For more information please contact the project website at:                *
*  http://campar.in.tum.de/Main/CovarianceEstimator                           *
*                                                                             *
*  The software package makes use of three additional libraries:              *
*  - The openCV computer vision library http://opencv.willowgarage.com/       *
*    published under the open source BSD license.                             *
*  - The SIFT implementation from Rob Hess <hess@eecs.oregonstate.edu>.       *
*    please see http://web.engr.oregonstate.edu/~hess/                        *
*    The package is distributed under GNU GPL v2 exept for the SIFT featur    *
*    detection code. For more information on SIFT licensing see LISENCE.ubc.  *
*  - The openSURF library from http://code.google.com/p/opensurf1/            *
*    The package is distributed under GNU GPL v3.                             *
*                                                                             *
*  If you use our software, please cite following publication:                *
*  Zeisl, B. et.al. "Estimation of Location Uncertainty for Scale Invariant   *
*    Feature Points", Proc. of the British Maschine Vision Conference (BMCV)  *
*    2009                                                                     *
*                                                                             *
*  @INPROCEEDINGS{ zeisl09elu,                                                *
*    AUTHOR={Bernhard Zeisl and Pierre Fite Georgel and Florian Schweiger and *
*            Eckehard Steinbach and Nassir Navab},                            *
*    TITEL={{Estimation of Location Uncertainty for Scale Invariant Feature   *
*            Points}},                                                        *
*    BOOKTITLE={Proc. of the British Maschine Vision Conference (BMCV)},      *
*    YEAR={2009}                                                              *
*  }                                                                          *
*                                                                             *
*******************************************************************************/

#pragma once

#include <iostream>
#include <string>

#include "definitions.h"

using namespace std;

class DecArgumentEvaluator {

public:
	/*** Constructor, Destructor **/
	DecArgumentEvaluator() :
		imgDir("./"), imgFile(NULL), decType(-1),
		verbose(true), showPts(false), savePts(false) {};

	DecArgumentEvaluator(int argc, char* argv[]) :
		decType(-1) {
		evaluate(argc, argv);
	};

	/*** Methods ***/
	void evaluate(int argc, char* argv[]);
	
	
	/*** Member variables ***/
	char* imgDir, * imgFile;
	char imgFileStart[25];
	string keyFile, decTypeName;
	int decType;
	bool verbose, showPts, savePts;

};


void DecArgumentEvaluator::evaluate(int argc, char *argv[]) {
	// information on usage
	if (argc==1) {
		cerr << "USAGE:\n"
		 << "  ./detector -t SURF|SIFT -i <img> [OPTIONS]\n"
		 << "\n"
		 << "REQUIRED PARAMETERS:\n"
		 << "  -t <type>		  Detector type, SURF or SIFT\n"
		 << "  -i <img>		      Image file name of image to process. PGM file format needed.\n"
		 << endl
		 << "OPTIONS:\n"
		 << "  -d <dir>           The directory containing the image file (default is ./)\n"
		 << "  -o <name>          Keypoint output file (default: IMG" << FE_KEYS_SIFT << "|" << FE_KEYS_SURF << ").\n"
		 << "  -q                 Quiet mode. Don't print status information.\n"
		 << "  -show              Display the found feature points in the image.\n"
		 << "  -save              Draw the points in the image and save to file.\n";

		exit(0);
	}

	// parse arguments
	int arg = 0;
	while (++arg < argc) { 
		if (! strcmp(argv[arg], "-t")) {
			decTypeName = argv[++arg];
			if(! strcmp(decTypeName.c_str(), "SIFT") )
				decType = DETECTOR_SIFT;
			else if(! strcmp(decTypeName.c_str(), "SURF") )
				decType = DETECTOR_SURF;
			else {
				cout << "DEC Error: Unknown detector type." << endl;
				exit(1);
			}
		}
		else if (! strcmp(argv[arg], "-d"))
			imgDir = argv[++arg];
		else if (! strcmp(argv[arg], "-i"))
			imgFile = argv[++arg]; 
		else if (! strcmp(argv[arg], "-q"))
			verbose = false;
		else if (! strcmp(argv[arg], "-o"))
			keyFile = argv[++arg];
		else if (! strcmp(argv[arg], "-show"))
			showPts = true;
		else if (! strcmp(argv[arg], "-save"))
			savePts = true;
		else
			if(verbose)
				cout << "DEC Warning: Skipping unknown parameter " << argv[arg] << "." << endl;
	}

	if(imgFile == NULL) {
		cout << "DEC Error: No image file specified." << endl;
		exit(1);
	}
	else {
		int i = 0;
		while(imgFile[i] != '.') imgFileStart[i] = imgFile[i++];
		imgFileStart[i] = 0;
	}
}