%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  --- COVARIANCE ESTIMATOR ---                                               %
%                                                                             %
%  Copyright 2009 Bernhard Zeisl, Pierre Fite Georgel                         %
%                                                                             %
%  This program is free software: you can redistribute it and/or modify it    %
%  under the terms of the GNU General Public License as published by the Free %
%  Software Foundation, version 3 of the License.                             %
%  This program is distributed in the hope that it will be useful, but        %
%  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY %
%  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License    %
%  (LISENCE.txt) and the disclaimer (DISCLAIMER.txt) for more details.        %
%                                                                             %
%  The software was written by Bernhard Zeisl <bernhard.zeisl@mytum.de and    %
%  Pierre Fite Georgel <pierre.georgel@gmail.com>.                            %
%  For more information please contact the project website at:                %
%  http://campar.in.tum.de/Main/CovarianceEstimator                           %
%                                                                             %
%  The software package makes use of three additional libraries:              %
%  - The openCV computer vision library http://opencv.willowgarage.com/       %
%    published under the open source BSD license.                             %
%  - The SIFT implementation from Rob Hess <hess@eecs.oregonstate.edu>.       %
%    please see http://web.engr.oregonstate.edu/~hess/                        %
%    The package is distributed under GNU GPL v2 exept for the SIFT featur    %
%    detection code. For more information on SIFT licensing see LISENCE.ubc.  %
%  - The openSURF library from http://code.google.com/p/opensurf1/            %
%    The package is distributed under GNU GPL v3.                             %
%                                                                             %
%  If you use our software, please cite following publication:                %
%  Zeisl, B. et.al. "Estimation of Location Uncertainty for Scale Invariant   %
%    Feature Points", Proc. of the British Maschine Vision Conference (BMCV)  %
%    2009                                                                     %
%                                                                             %
%  @INPROCEEDINGS{ zeisl09elu,                                                %
%    AUTHOR={Bernhard Zeisl and Pierre Fite Georgel and Florian Schweiger and %
%            Eckehard Steinbach and Nassir Navab},                            %
%    TITEL={{Estimation of Location Uncertainty for Scale Invariant Feature   %
%            Points}},                                                        %
%    BOOKTITLE={Proc. of the British Maschine Vision Conference (BMCV)},      %
%    YEAR={2009}                                                              %
%  }                                                                          %
%                                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%/


% Class for storing keypoint and covariance information
% class structure in accordance with the C/C++ file GeneralFeature.h
classdef GeneralFeature
    properties
        x
        y
        scale
        orientation
        laplacian
        cov
        descrLength
        descriptor
        detectorType
    end
    
    
    methods (Access = 'public')
        % constructor
        function this = GeneralFeature(x,y,scl,ori,lap,cov,descr,type)
            this.x = x;
            this.y = y;
            this.scale = scl;
            this.orientation = ori;
            this.laplacian = lap;
            this.cov = cov;
            this.descriptor = descr;
            this.descrLength = length(descr);
            this.detectorType = type;            
        end
    end
    
    methods (Static)
        % read keys from file
        function [keys] = readFrom(filename)
            % open file
            fid = fopen(filename, 'r');
            if(~fid)
               disp 'File could not be read'; 
            end

            % read file format
            GeneralFeature.skipComments(fid);
            type = fgetl(fid);
            vals = fscanf(fid, '%i\t%i');
            nrPoints = vals(1);
            descrLen = vals(2);

            % read the point data
            GeneralFeature.skipComments(fid);
            i = 1;
            while i <= nrPoints
                % read the position, scale etc
                %                      x   y   s   o   l  cx  cxy cy
                % vals = fscanf(fid, '%f\t%f\t%f\t%f\t%i\t%f\t%f\t%f');
                vals = textscan(fid, '%f\t', 8);
                vals = vals{1};
                x = vals(1); y = vals(2);
                scl = vals(3); ori = vals(4); lap = vals(5);
                cov = [vals(6) vals(7); vals(7), vals(8)];

                % read the descriptor if specified
                if descrLen > 0
                    descr = textscan(fid, '\t%f', descrLen);
                end

                % add key to array of keys
                keys(i) = GeneralFeature(x,y,scl,ori,lap,cov,descr{1},type);

                i = i+1
            end
        end

        % skip all line beginning with '#'
        function skipComments(fid)
            line = '#';
            while line(1) == '#'
                position = ftell(fid);
                line = fgets(fid);
            end
            fseek(fid, position, 'bof');
        end

    end   
end
